"""**Vector store** stores embedded data and performs vector search.

One of the most common ways to store and search over unstructured data is to
embed it and store the resulting embedding vectors, and then query the store
and retrieve the data that are 'most similar' to the embedded query.

**Class hierarchy:**

.. code-block::

    VectorStore --> <name>  # Examples: Annoy, FAISS, Milvus

    BaseRetriever --> VectorStoreRetriever --> <name>Retriever  # Example: VespaRetriever

**Main helpers:**

.. code-block::

    Embeddings, Document
"""  # noqa: E501
from langchaincoexpert.vectorstores.alibabacloud_opensearch import (
    AlibabaCloudOpenSearch,
    AlibabaCloudOpenSearchSettings,
)
from langchaincoexpert.vectorstores.analyticdb import AnalyticDB
from langchaincoexpert.vectorstores.annoy import Annoy
from langchaincoexpert.vectorstores.atlas import AtlasDB
from langchaincoexpert.vectorstores.awadb import AwaDB
from langchaincoexpert.vectorstores.azuresearch import AzureSearch
from langchaincoexpert.vectorstores.bageldb import Bagel
from langchaincoexpert.vectorstores.base import VectorStore
from langchaincoexpert.vectorstores.cassandra import Cassandra
from langchaincoexpert.vectorstores.chroma import Chroma
from langchaincoexpert.vectorstores.clarifai import Clarifai
from langchaincoexpert.vectorstores.clickhouse import Clickhouse, ClickhouseSettings
from langchaincoexpert.vectorstores.dashvector import DashVector
from langchaincoexpert.vectorstores.deeplake import DeepLake
from langchaincoexpert.vectorstores.dingo import Dingo
from langchaincoexpert.vectorstores.docarray import DocArrayHnswSearch, DocArrayInMemorySearch
from langchaincoexpert.vectorstores.elastic_vector_search import (
    ElasticKnnSearch,
    ElasticVectorSearch,
)
from langchaincoexpert.vectorstores.elasticsearch import ElasticsearchStore
from langchaincoexpert.vectorstores.epsilla import Epsilla
from langchaincoexpert.vectorstores.faiss import FAISS
from langchaincoexpert.vectorstores.hologres import Hologres
from langchaincoexpert.vectorstores.lancedb import LanceDB
from langchaincoexpert.vectorstores.marqo import Marqo
from langchaincoexpert.vectorstores.matching_engine import MatchingEngine
from langchaincoexpert.vectorstores.meilisearch import Meilisearch
from langchaincoexpert.vectorstores.milvus import Milvus
from langchaincoexpert.vectorstores.mongodb_atlas import MongoDBAtlasVectorSearch
from langchaincoexpert.vectorstores.myscale import MyScale, MyScaleSettings
from langchaincoexpert.vectorstores.neo4j_vector import Neo4jVector
from langchaincoexpert.vectorstores.opensearch_vector_search import OpenSearchVectorSearch
from langchaincoexpert.vectorstores.pgembedding import PGEmbedding
from langchaincoexpert.vectorstores.pgvector import PGVector
from langchaincoexpert.vectorstores.pinecone import Pinecone
from langchaincoexpert.vectorstores.qdrant import Qdrant
from langchaincoexpert.vectorstores.redis import Redis
from langchaincoexpert.vectorstores.rocksetdb import Rockset
from langchaincoexpert.vectorstores.scann import ScaNN
from langchaincoexpert.vectorstores.singlestoredb import SingleStoreDB
from langchaincoexpert.vectorstores.sklearn import SKLearnVectorStore
from langchaincoexpert.vectorstores.sqlitevss import SQLiteVSS
from langchaincoexpert.vectorstores.starrocks import StarRocks
from langchaincoexpert.vectorstores.supabase import SupabaseVectorStore
from langchaincoexpert.vectorstores.tair import Tair
from langchaincoexpert.vectorstores.tencentvectordb import TencentVectorDB
from langchaincoexpert.vectorstores.tigris import Tigris
from langchaincoexpert.vectorstores.typesense import Typesense
from langchaincoexpert.vectorstores.usearch import USearch
from langchaincoexpert.vectorstores.vectara import Vectara
from langchaincoexpert.vectorstores.weaviate import Weaviate
from langchaincoexpert.vectorstores.zep import ZepVectorStore
from langchaincoexpert.vectorstores.zilliz import Zilliz

__all__ = [
    "AlibabaCloudOpenSearch",
    "AlibabaCloudOpenSearchSettings",
    "AnalyticDB",
    "Annoy",
    "Annoy",
    "AtlasDB",
    "AtlasDB",
    "AwaDB",
    "AzureSearch",
    "Bagel",
    "Cassandra",
    "Chroma",
    "Chroma",
    "Clarifai",
    "Clickhouse",
    "ClickhouseSettings",
    "DashVector",
    "DeepLake",
    "DeepLake",
    "Dingo",
    "DocArrayHnswSearch",
    "DocArrayInMemorySearch",
    "ElasticKnnSearch",
    "ElasticVectorSearch",
    "ElasticsearchStore",
    "Epsilla",
    "FAISS",
    "Hologres",
    "LanceDB",
    "Marqo",
    "MatchingEngine",
    "Meilisearch",
    "Milvus",
    "MongoDBAtlasVectorSearch",
    "MyScale",
    "MyScaleSettings",
    "Neo4jVector",
    "OpenSearchVectorSearch",
    "OpenSearchVectorSearch",
    "PGEmbedding",
    "PGVector",
    "Pinecone",
    "Qdrant",
    "Redis",
    "Rockset",
    "SKLearnVectorStore",
    "ScaNN",
    "SingleStoreDB",
    "SingleStoreDB",
    "SQLiteVSS",
    "StarRocks",
    "SupabaseVectorStore",
    "Tair",
    "Tigris",
    "Typesense",
    "USearch",
    "Vectara",
    "VectorStore",
    "Weaviate",
    "ZepVectorStore",
    "Zilliz",
    "Zilliz",
    "TencentVectorDB",
]



"""Retriever that generates and executes structured queries over its own data source."""

from typing import Any, Dict, List, Optional, Type, cast

from langchaincoexpert.callbacks.manager import CallbackManagerForRetrieverRun
from langchaincoexpert.chains import LLMChain
from langchaincoexpert.chains.query_constructor.base import load_query_constructor_chain
from langchaincoexpert.chains.query_constructor.ir import StructuredQuery, Visitor
from langchaincoexpert.chains.query_constructor.schema import AttributeInfo
from langchaincoexpert.pydantic_v1 import BaseModel, Field, root_validator
from langchaincoexpert.retrievers.self_query.chroma import ChromaTranslator
from langchaincoexpert.retrievers.self_query.dashvector import DashvectorTranslator
from langchaincoexpert.retrievers.self_query.deeplake import DeepLakeTranslator
from langchaincoexpert.retrievers.self_query.elasticsearch import ElasticsearchTranslator
from langchaincoexpert.retrievers.self_query.milvus import MilvusTranslator
from langchaincoexpert.retrievers.self_query.myscale import MyScaleTranslator
from langchaincoexpert.retrievers.self_query.pinecone import PineconeTranslator
from langchaincoexpert.retrievers.self_query.qdrant import QdrantTranslator
from langchaincoexpert.retrievers.self_query.redis import RedisTranslator
from langchaincoexpert.retrievers.self_query.supabase import SupabaseVectorTranslator
from langchaincoexpert.retrievers.self_query.vectara import VectaraTranslator
from langchaincoexpert.retrievers.self_query.weaviate import WeaviateTranslator
from langchaincoexpert.schema import BaseRetriever, Document
from langchaincoexpert.schema.language_model import BaseLanguageModel
from langchaincoexpert.vectorstores import (
    Chroma,
    DashVector,
    DeepLake,
    ElasticsearchStore,
    Milvus,
    MyScale,
    Pinecone,
    Qdrant,
    Redis,
    SupabaseVectorStore,
    Vectara,
    VectorStore,
    Weaviate,
)


def _get_builtin_translator(vectorstore: VectorStore) -> Visitor:
    """Get the translator class corresponding to the vector store class."""
    BUILTIN_TRANSLATORS: Dict[Type[VectorStore], Type[Visitor]] = {
        Pinecone: PineconeTranslator,
        Chroma: ChromaTranslator,
        DashVector: DashvectorTranslator,
        Weaviate: WeaviateTranslator,
        Vectara: VectaraTranslator,
        Qdrant: QdrantTranslator,
        MyScale: MyScaleTranslator,
        DeepLake: DeepLakeTranslator,
        ElasticsearchStore: ElasticsearchTranslator,
        Milvus: MilvusTranslator,
        SupabaseVectorStore: SupabaseVectorTranslator,
    }
    if isinstance(vectorstore, Qdrant):
        return QdrantTranslator(metadata_key=vectorstore.metadata_payload_key)
    elif isinstance(vectorstore, MyScale):
        return MyScaleTranslator(metadata_key=vectorstore.metadata_column)
    elif isinstance(vectorstore, Redis):
        return RedisTranslator.from_vectorstore(vectorstore)
    elif vectorstore.__class__ in BUILTIN_TRANSLATORS:
        return BUILTIN_TRANSLATORS[vectorstore.__class__]()
    else:
        raise ValueError(
            f"Self query retriever with Vector Store type {vectorstore.__class__}"
            f" not supported."
        )


class SelfQueryRetriever(BaseRetriever, BaseModel):
    """Retriever that uses a vector store and an LLM to generate
    the vector store queries."""

    vectorstore: VectorStore
    """The underlying vector store from which documents will be retrieved."""
    llm_chain: LLMChain
    """The LLMChain for generating the vector store queries."""
    search_type: str = "similarity"
    """The search type to perform on the vector store."""
    search_kwargs: dict = Field(default_factory=dict)
    """Keyword arguments to pass in to the vector store search."""
    structured_query_translator: Visitor
    """Translator for turning internal query language into vectorstore search params."""
    verbose: bool = False

    use_original_query: bool = False
    """Use original query instead of the revised new query from LLM"""

    class Config:
        """Configuration for this pydantic object."""

        arbitrary_types_allowed = True

    @root_validator(pre=True)
    def validate_translator(cls, values: Dict) -> Dict:
        """Validate translator."""
        if "structured_query_translator" not in values:
            values["structured_query_translator"] = _get_builtin_translator(
                values["vectorstore"]
            )
        return values

    def _get_relevant_documents(
        self, query: str, *, run_manager: CallbackManagerForRetrieverRun
    ) -> List[Document]:
        """Get documents relevant for a query.

        Args:
            query: string to find relevant documents for

        Returns:
            List of relevant documents
        """
        inputs = self.llm_chain.prep_inputs({"query": query})
        structured_query = cast(
            StructuredQuery,
            self.llm_chain.predict_and_parse(
                callbacks=run_manager.get_child(), **inputs
            ),
        )
        if self.verbose:
            print(structured_query)
        new_query, new_kwargs = self.structured_query_translator.visit_structured_query(
            structured_query
        )
        if structured_query.limit is not None:
            new_kwargs["k"] = structured_query.limit

        if self.use_original_query:
            new_query = query

        search_kwargs = {**self.search_kwargs, **new_kwargs}
        docs = self.vectorstore.search(new_query, self.search_type, **search_kwargs)
        return docs

    @classmethod
    def from_llm(
        cls,
        llm: BaseLanguageModel,
        vectorstore: VectorStore,
        document_contents: str,
        metadata_field_info: List[AttributeInfo],
        structured_query_translator: Optional[Visitor] = None,
        chain_kwargs: Optional[Dict] = None,
        enable_limit: bool = False,
        use_original_query: bool = False,
        **kwargs: Any,
    ) -> "SelfQueryRetriever":
        if structured_query_translator is None:
            structured_query_translator = _get_builtin_translator(vectorstore)
        chain_kwargs = chain_kwargs or {}

        if "allowed_comparators" not in chain_kwargs:
            chain_kwargs[
                "allowed_comparators"
            ] = structured_query_translator.allowed_comparators
        if "allowed_operators" not in chain_kwargs:
            chain_kwargs[
                "allowed_operators"
            ] = structured_query_translator.allowed_operators
        llm_chain = load_query_constructor_chain(
            llm,
            document_contents,
            metadata_field_info,
            enable_limit=enable_limit,
            **chain_kwargs,
        )
        return cls(
            llm_chain=llm_chain,
            vectorstore=vectorstore,
            use_original_query=use_original_query,
            structured_query_translator=structured_query_translator,
            **kwargs,
        )



"""
**LLM** classes provide
access to the large language model (**LLM**) APIs and services.

**Class hierarchy:**

.. code-block::

    BaseLanguageModel --> BaseLLM --> LLM --> <name>  # Examples: AI21, HuggingFaceHub, OpenAI

**Main helpers:**

.. code-block::

    LLMResult, PromptValue,
    CallbackManagerForLLMRun, AsyncCallbackManagerForLLMRun,
    CallbackManager, AsyncCallbackManager,
    AIMessage, BaseMessage
"""  # noqa: E501
from typing import Dict, Type

from langchaincoexpert.llms.ai21 import AI21
from langchaincoexpert.llms.aleph_alpha import AlephAlpha
from langchaincoexpert.llms.amazon_api_gateway import AmazonAPIGateway
from langchaincoexpert.llms.anthropic import Anthropic
from langchaincoexpert.llms.anyscale import Anyscale
from langchaincoexpert.llms.aviary import Aviary
from langchaincoexpert.llms.azureml_endpoint import AzureMLOnlineEndpoint
from langchaincoexpert.llms.baidu_qianfan_endpoint import QianfanLLMEndpoint
from langchaincoexpert.llms.bananadev import Banana
from langchaincoexpert.llms.base import BaseLLM
from langchaincoexpert.llms.baseten import Baseten
from langchaincoexpert.llms.beam import Beam
from langchaincoexpert.llms.bedrock import Bedrock
from langchaincoexpert.llms.bittensor import NIBittensorLLM
from langchaincoexpert.llms.cerebriumai import CerebriumAI
from langchaincoexpert.llms.chatglm import ChatGLM
from langchaincoexpert.llms.clarifai import Clarifai
from langchaincoexpert.llms.cohere import Cohere
from langchaincoexpert.llms.ctransformers import CTransformers
from langchaincoexpert.llms.ctranslate2 import CTranslate2
from langchaincoexpert.llms.databricks import Databricks
from langchaincoexpert.llms.deepinfra import DeepInfra
from langchaincoexpert.llms.deepsparse import DeepSparse
from langchaincoexpert.llms.edenai import EdenAI
from langchaincoexpert.llms.fake import FakeListLLM
from langchaincoexpert.llms.fireworks import Fireworks, FireworksChat
from langchaincoexpert.llms.forefrontai import ForefrontAI
from langchaincoexpert.llms.google_palm import GooglePalm
from langchaincoexpert.llms.gooseai import GooseAI
from langchaincoexpert.llms.gpt4all import GPT4All
from langchaincoexpert.llms.huggingface_endpoint import HuggingFaceEndpoint
from langchaincoexpert.llms.huggingface_hub import HuggingFaceHub
from langchaincoexpert.llms.huggingface_pipeline import HuggingFacePipeline
from langchaincoexpert.llms.huggingface_text_gen_inference import HuggingFaceTextGenInference
from langchaincoexpert.llms.human import HumanInputLLM
from langchaincoexpert.llms.koboldai import KoboldApiLLM
from langchaincoexpert.llms.llamacpp import LlamaCpp
from langchaincoexpert.llms.manifest import ManifestWrapper
from langchaincoexpert.llms.minimax import Minimax
from langchaincoexpert.llms.mlflow_ai_gateway import MlflowAIGateway
from langchaincoexpert.llms.modal import Modal
from langchaincoexpert.llms.mosaicml import MosaicML
from langchaincoexpert.llms.nlpcloud import NLPCloud
from langchaincoexpert.llms.octoai_endpoint import OctoAIEndpoint
from langchaincoexpert.llms.ollama import Ollama
from langchaincoexpert.llms.opaqueprompts import OpaquePrompts
from langchaincoexpert.llms.openai import AzureOpenAI, OpenAI, OpenAIChat
from langchaincoexpert.llms.openllm import OpenLLM
from langchaincoexpert.llms.openlm import OpenLM
from langchaincoexpert.llms.petals import Petals
from langchaincoexpert.llms.pipelineai import PipelineAI
from langchaincoexpert.llms.predibase import Predibase
from langchaincoexpert.llms.predictionguard import PredictionGuard
from langchaincoexpert.llms.promptlayer_openai import PromptLayerOpenAI, PromptLayerOpenAIChat
from langchaincoexpert.llms.replicate import Replicate
from langchaincoexpert.llms.rwkv import RWKV
from langchaincoexpert.llms.sagemaker_endpoint import SagemakerEndpoint
from langchaincoexpert.llms.self_hosted import SelfHostedPipeline
from langchaincoexpert.llms.self_hosted_hugging_face import SelfHostedHuggingFaceLLM
from langchaincoexpert.llms.stochasticai import StochasticAI
from langchaincoexpert.llms.symblai_nebula import Nebula
from langchaincoexpert.llms.textgen import TextGen
from langchaincoexpert.llms.titan_takeoff import TitanTakeoff
from langchaincoexpert.llms.tongyi import Tongyi
from langchaincoexpert.llms.vertexai import VertexAI, VertexAIModelGarden
from langchaincoexpert.llms.vllm import VLLM, VLLMOpenAI
from langchaincoexpert.llms.writer import Writer
from langchaincoexpert.llms.xinference import Xinference

__all__ = [
    "AI21",
    "AlephAlpha",
    "AmazonAPIGateway",
    "Anthropic",
    "Anyscale",
    "Aviary",
    "AzureMLOnlineEndpoint",
    "AzureOpenAI",
    "Banana",
    "Baseten",
    "Beam",
    "Bedrock",
    "CTransformers",
    "CTranslate2",
    "CerebriumAI",
    "ChatGLM",
    "Clarifai",
    "Cohere",
    "Databricks",
    "DeepInfra",
    "DeepSparse",
    "EdenAI",
    "FakeListLLM",
    "Fireworks",
    "FireworksChat",
    "ForefrontAI",
    "GPT4All",
    "GooglePalm",
    "GooseAI",
    "HuggingFaceEndpoint",
    "HuggingFaceHub",
    "HuggingFacePipeline",
    "HuggingFaceTextGenInference",
    "HumanInputLLM",
    "KoboldApiLLM",
    "LlamaCpp",
    "TextGen",
    "ManifestWrapper",
    "Minimax",
    "MlflowAIGateway",
    "Modal",
    "MosaicML",
    "Nebula",
    "NIBittensorLLM",
    "NLPCloud",
    "Ollama",
    "OpenAI",
    "OpenAIChat",
    "OpenLLM",
    "OpenLM",
    "Petals",
    "PipelineAI",
    "Predibase",
    "PredictionGuard",
    "PromptLayerOpenAI",
    "PromptLayerOpenAIChat",
    "OpaquePrompts",
    "RWKV",
    "Replicate",
    "SagemakerEndpoint",
    "SelfHostedHuggingFaceLLM",
    "SelfHostedPipeline",
    "StochasticAI",
    "TitanTakeoff",
    "Tongyi",
    "VertexAI",
    "VertexAIModelGarden",
    "VLLM",
    "VLLMOpenAI",
    "Writer",
    "OctoAIEndpoint",
    "Xinference",
    "QianfanLLMEndpoint",
]

type_to_cls_dict: Dict[str, Type[BaseLLM]] = {
    "ai21": AI21,
    "aleph_alpha": AlephAlpha,
    "amazon_api_gateway": AmazonAPIGateway,
    "amazon_bedrock": Bedrock,
    "anthropic": Anthropic,
    "anyscale": Anyscale,
    "aviary": Aviary,
    "azure": AzureOpenAI,
    "azureml_endpoint": AzureMLOnlineEndpoint,
    "bananadev": Banana,
    "baseten": Baseten,
    "beam": Beam,
    "cerebriumai": CerebriumAI,
    "chat_glm": ChatGLM,
    "clarifai": Clarifai,
    "cohere": Cohere,
    "ctransformers": CTransformers,
    "ctranslate2": CTranslate2,
    "databricks": Databricks,
    "deepinfra": DeepInfra,
    "deepsparse": DeepSparse,
    "edenai": EdenAI,
    "fake-list": FakeListLLM,
    "forefrontai": ForefrontAI,
    "google_palm": GooglePalm,
    "gooseai": GooseAI,
    "gpt4all": GPT4All,
    "huggingface_endpoint": HuggingFaceEndpoint,
    "huggingface_hub": HuggingFaceHub,
    "huggingface_pipeline": HuggingFacePipeline,
    "huggingface_textgen_inference": HuggingFaceTextGenInference,
    "human-input": HumanInputLLM,
    "koboldai": KoboldApiLLM,
    "llamacpp": LlamaCpp,
    "textgen": TextGen,
    "minimax": Minimax,
    "mlflow-ai-gateway": MlflowAIGateway,
    "modal": Modal,
    "mosaic": MosaicML,
    "nebula": Nebula,
    "nibittensor": NIBittensorLLM,
    "nlpcloud": NLPCloud,
    "ollama": Ollama,
    "openai": OpenAI,
    "openlm": OpenLM,
    "petals": Petals,
    "pipelineai": PipelineAI,
    "predibase": Predibase,
    "opaqueprompts": OpaquePrompts,
    "replicate": Replicate,
    "rwkv": RWKV,
    "sagemaker_endpoint": SagemakerEndpoint,
    "self_hosted": SelfHostedPipeline,
    "self_hosted_hugging_face": SelfHostedHuggingFaceLLM,
    "stochasticai": StochasticAI,
    "tongyi": Tongyi,
    "titan_takeoff": TitanTakeoff,
    "vertexai": VertexAI,
    "vertexai_model_garden": VertexAIModelGarden,
    "openllm": OpenLLM,
    "openllm_client": OpenLLM,
    "vllm": VLLM,
    "vllm_openai": VLLMOpenAI,
    "writer": Writer,
    "xinference": Xinference,
    "qianfan_endpoint": QianfanLLMEndpoint,
}



from __future__ import annotations

import os
import time
import uuid
from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Tuple, Type

import numpy as np

from langchaincoexpert.docstore.document import Document
from langchaincoexpert.embeddings.base import Embeddings
from langchaincoexpert.vectorstores.base import VectorStore

if TYPE_CHECKING:
    import vearch
DEFAULT_TOPN = 4


class VearchDb(VectorStore):
    _DEFAULT_TABLE_NAME = "langchaincoexpert_vearch"

    def __init__(
        self,
        embedding_function: Embeddings,
        table_name: str = _DEFAULT_TABLE_NAME,
        metadata_path: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """Initialize vearch vector store"""
        try:
            import vearch
        except ImportError:
            raise ValueError(
                "Could not import vearch python package. "
                "Please install it with `pip install vearch`."
            )

        if metadata_path is None:
            metadata_path = os.getcwd().replace("\\", "/")
        if not os.path.isdir(metadata_path):
            os.makedirs(metadata_path)
        log_path = os.path.join(metadata_path, "log")
        if not os.path.isdir(log_path):
            os.makedirs(log_path)
        self.vearch_engine = vearch.Engine(metadata_path, log_path)

        if not table_name:
            table_name = self._DEFAULT_TABLE_NAME
            table_name += "_"
            table_name += str(uuid.uuid4()).split("-")[-1]
        self.using_table_name = table_name
        self.using_metapath = metadata_path
        self.embedding_func = embedding_function

    @property
    def embeddings(self) -> Optional[Embeddings]:
        return self.embedding_func

    @classmethod
    def from_documents(
        cls: Type[VearchDb],
        documents: List[Document],
        embedding: Embeddings,
        table_name: str = "langchaincoexpert_vearch",
        metadata_path: Optional[str] = None,
        **kwargs: Any,
    ) -> VearchDb:
        """Return Vearch VectorStore"""

        texts = [d.page_content for d in documents]
        metadatas = [d.metadata for d in documents]

        return cls.from_texts(
            texts=texts,
            embedding=embedding,
            metadatas=metadatas,
            table_name=table_name,
            metadata_path=metadata_path,
            **kwargs,
        )

    @classmethod
    def from_texts(
        cls: Type[VearchDb],
        texts: List[str],
        embedding: Embeddings,
        metadatas: Optional[List[dict]] = None,
        table_name: str = _DEFAULT_TABLE_NAME,
        metadata_path: Optional[str] = None,
        **kwargs: Any,
    ) -> VearchDb:
        """Return Vearch VectorStore"""

        vearch_db = cls(
            embedding_function=embedding,
            table_name=table_name,
            metadata_path=metadata_path,
        )
        vearch_db.add_texts(texts=texts, metadatas=metadatas)
        return vearch_db

    def _create_table(
        self,
        dim: int = 1024,
        filed_list: List[dict] = [
            {"filed": "text", "type": "str"},
            {"filed": "metadata", "type": "str"},
        ],
    ) -> int:
        """
        Create VectorStore Table
        Args:
            dim:dimension of vector
            fileds_list: the filed you want to store
        Return:
            code,0 for success,1 for failed
        """
        type_dict = {"int": vearch.dataType.INT, "str": vearch.dataType.STRING}
        engine_info = {
            "index_size": 10000,
            "retrieval_type": "IVFPQ",
            "retrieval_param": {"ncentroids": 2048, "nsubvector": 32},
        }
        fields = [
            vearch.GammaFieldInfo(fi["filed"], type_dict[fi["type"]])
            for fi in filed_list
        ]
        vector_field = vearch.GammaVectorInfo(
            name="text_embedding",
            type=vearch.dataType.VECTOR,
            is_index=True,
            dimension=dim,
            model_id="",
            store_type="MemoryOnly",
            store_param={"cache_size": 10000},
            has_source=False,
        )
        response_code = self.vearch_engine.create_table(
            engine_info,
            name=self.using_table_name,
            fields=fields,
            vector_field=vector_field,
        )
        return response_code

    def add_texts(
        self,
        texts: Iterable[str],
        metadatas: Optional[List[dict]] = None,
        **kwargs: Any,
    ) -> List[str]:
        """
        Returns:
            List of ids from adding the texts into the vectorstore.
        """
        embeddings = None
        if self.embedding_func is not None:
            embeddings = self.embedding_func.embed_documents(list(texts))
        table_path = os.path.join(
            self.using_metapath, self.using_table_name + ".schema"
        )
        if not os.path.exists(table_path):
            if embeddings is None:
                raise ValueError("embeddings is None")
            dim = len(embeddings[0])
            response_code = self._create_table(dim)
            if response_code:
                raise ValueError("create table failed!!!")
        if embeddings is not None and metadatas is not None:
            doc_items = []
            for text, metadata, embed in zip(texts, metadatas, embeddings):
                profiles: dict[str, Any] = {}
                profiles["text"] = text
                profiles["metadata"] = metadata["source"]
                profiles["text_embedding"] = embed
                doc_items.append(profiles)

            docid = self.vearch_engine.add(doc_items)
            t_time = 0
            while len(docid) != len(embeddings):
                time.sleep(0.5)
                if t_time > 6:
                    break
                t_time += 1
            self.vearch_engine.dump()
        return docid

    def _load(self) -> None:
        """
        load vearch engine
        """
        self.vearch_engine.load()

    @classmethod
    def load_local(
        cls,
        embedding: Embeddings,
        table_name: str = _DEFAULT_TABLE_NAME,
        metadata_path: Optional[str] = None,
        **kwargs: Any,
    ) -> VearchDb:
        """Load the local specified table.
        Returns:
            Success or failure of loading the local specified table
        """
        if not metadata_path:
            raise ValueError("No metadata path!!!")
        if not table_name:
            raise ValueError("No table name!!!")
        table_path = os.path.join(metadata_path, table_name + ".schema")
        if not os.path.exists(table_path):
            raise ValueError("vearch vectorbase table not exist!!!")
        vearch_db = cls(
            embedding_function=embedding,
            table_name=table_name,
            metadata_path=metadata_path,
        )
        vearch_db._load()
        return vearch_db

    def similarity_search(
        self,
        query: str,
        k: int = DEFAULT_TOPN,
        **kwargs: Any,
    ) -> List[Document]:
        """
        Return docs most similar to query.

        """
        if self.vearch_engine is None:
            raise ValueError("Vearch engine is None!!!")
        if self.embedding_func is None:
            raise ValueError("embedding_func is None!!!")
        embeddings = self.embedding_func.embed_query(query)
        docs = self.similarity_search_by_vector(embeddings, k)
        return docs

    def similarity_search_by_vector(
        self,
        embedding: List[float],
        k: int = DEFAULT_TOPN,
        **kwargs: Any,
    ) -> List[Document]:
        """The most k similar documents and scores of the specified query.

        Args:
            embeddings: embedding vector of the query.
            k: The k most similar documents to the text query.
            min_score: the score of similar documents to the text query
        Returns:
            The k most similar documents to the specified text query.
            0 is dissimilar, 1 is the most similar.
        """
        query_data = {
            "vector": [
                {
                    "field": "text_embedding",
                    "feature": np.array(embedding),
                }
            ],
            "fields": [],
            "is_brute_search": 1,
            "retrieval_param": {"metric_type": "InnerProduct", "nprobe": 20},
            "topn": k,
        }
        query_result = self.vearch_engine.search(query_data)
        docs = []
        for item in query_result[0]["result_items"]:
            content = ""
            meta_data = {}
            for item_key in item:
                if item_key == "text":
                    content = item[item_key]
                    continue
                if item_key == "metadata":
                    meta_data["source"] = item[item_key]
                    continue
            docs.append(Document(page_content=content, metadata=meta_data))
        return docs

    def similarity_search_with_score(
        self,
        query: str,
        k: int = DEFAULT_TOPN,
        **kwargs: Any,
    ) -> List[Tuple[Document, float]]:
        """The most k similar documents and scores of the specified query.

        Args:
            embeddings: embedding vector of the query.
            k: The k most similar documents to the text query.
            min_score: the score of similar documents to the text query
        Returns:
            The k most similar documents to the specified text query.
            0 is dissimilar, 1 is the most similar.
        """
        if self.embedding_func is None:
            raise ValueError("embedding_func is None!!!")
        embeddings = self.embedding_func.embed_query(query)
        query_data = {
            "vector": [
                {
                    "field": "text_embedding",
                    "feature": np.array(embeddings),
                }
            ],
            "fields": [],
            "is_brute_search": 1,
            "retrieval_param": {"metric_type": "InnerProduct", "nprobe": 20},
            "topn": k,
        }
        query_result = self.vearch_engine.search(query_data)
        results: List[Tuple[Document, float]] = []
        for item in query_result[0]["result_items"]:
            content = ""
            meta_data = {}
            for item_key in item:
                if item_key == "text":
                    content = item[item_key]
                    continue
                if item_key == "metadata":
                    meta_data["source"] = item[item_key]
                    continue
                if item_key == "score":
                    score = item[item_key]
                    continue
            tmp_res = (Document(page_content=content, metadata=meta_data), score)
            results.append(tmp_res)
        return results

    def _similarity_search_with_relevance_scores(
        self,
        query: str,
        k: int = 4,
        **kwargs: Any,
    ) -> List[Tuple[Document, float]]:
        return self.similarity_search_with_score(query, k, **kwargs)

    def delete(
        self,
        ids: Optional[List[str]] = None,
        **kwargs: Any,
    ) -> Optional[bool]:
        """Delete the documents which have the specified ids.

        Args:
            ids: The ids of the embedding vectors.
            **kwargs: Other keyword arguments that subclasses might use.
        Returns:
            Optional[bool]: True if deletion is successful.
            False otherwise, None if not implemented.
        """
        if self.vearch_engine is None:
            raise ValueError("Verach Engine is None!!!")
        ret: Optional[bool] = None
        tmp_res = []
        if ids is None or ids.__len__() == 0:
            return ret
        for _id in ids:
            ret = self.vearch_engine.del_doc(_id)
            tmp_res.append(ret)
        ret = all(i == 0 for i in tmp_res)
        return ret

    def get(
        self,
        ids: Optional[List[str]] = None,
        **kwargs: Any,
    ) -> Dict[str, Document]:
        """Return docs according ids.

        Args:
            ids: The ids of the embedding vectors.
        Returns:
            Documents which satisfy the input conditions.
        """

        if self.vearch_engine is None:
            raise ValueError("vearch engine is None!!!")
        results: Dict[str, Document] = {}
        if ids is None or ids.__len__() == 0:
            return results
        for id in ids:
            docs_detail = self.vearch_engine.get_doc_by_id(id)
            if docs_detail == {}:
                continue

            content = ""
            meta_info = {}
            for field in docs_detail:
                if field == "text":
                    content = docs_detail[field]
                    continue
                elif field == "metadata":
                    meta_info["source"] = docs_detail[field]
                    continue
            results[docs_detail["_id"]] = Document(
                page_content=content, metadata=meta_info
            )
        return results



"""Tracers that record execution of langchaincoexpert runs."""

from langchaincoexpert.callbacks.tracers.langchain import langchaincoexpertTracer
from langchaincoexpert.callbacks.tracers.langchain_v1 import langchaincoexpertTracerV1
from langchaincoexpert.callbacks.tracers.stdout import (
    ConsoleCallbackHandler,
    FunctionCallbackHandler,
)
from langchaincoexpert.callbacks.tracers.wandb import WandbTracer

__all__ = [
    "langchaincoexpertTracer",
    "langchaincoexpertTracerV1",
    "FunctionCallbackHandler",
    "ConsoleCallbackHandler",
    "WandbTracer",
]



"""**Embedding models**  are wrappers around embedding models
from different APIs and services.

**Embedding models** can be LLMs or not.

**Class hierarchy:**

.. code-block::

    Embeddings --> <name>Embeddings  # Examples: OpenAIEmbeddings, HuggingFaceEmbeddings
"""


import logging
from typing import Any

from langchaincoexpert.embeddings.aleph_alpha import (
    AlephAlphaAsymmetricSemanticEmbedding,
    AlephAlphaSymmetricSemanticEmbedding,
)
from langchaincoexpert.embeddings.awa import AwaEmbeddings
from langchaincoexpert.embeddings.baidu_qianfan_endpoint import QianfanEmbeddingsEndpoint
from langchaincoexpert.embeddings.bedrock import BedrockEmbeddings
from langchaincoexpert.embeddings.cache import CacheBackedEmbeddings
from langchaincoexpert.embeddings.clarifai import ClarifaiEmbeddings
from langchaincoexpert.embeddings.cohere import CohereEmbeddings
from langchaincoexpert.embeddings.dashscope import DashScopeEmbeddings
from langchaincoexpert.embeddings.deepinfra import DeepInfraEmbeddings
from langchaincoexpert.embeddings.edenai import EdenAiEmbeddings
from langchaincoexpert.embeddings.elasticsearch import ElasticsearchEmbeddings
from langchaincoexpert.embeddings.embaas import EmbaasEmbeddings
from langchaincoexpert.embeddings.ernie import ErnieEmbeddings
from langchaincoexpert.embeddings.fake import DeterministicFakeEmbedding, FakeEmbeddings
from langchaincoexpert.embeddings.google_palm import GooglePalmEmbeddings
from langchaincoexpert.embeddings.gpt4all import GPT4AllEmbeddings
from langchaincoexpert.embeddings.huggingface import (
    HuggingFaceBgeEmbeddings,
    HuggingFaceEmbeddings,
    HuggingFaceInferenceAPIEmbeddings,
    HuggingFaceInstructEmbeddings,
)
from langchaincoexpert.embeddings.huggingface_hub import HuggingFaceHubEmbeddings
from langchaincoexpert.embeddings.jina import JinaEmbeddings
from langchaincoexpert.embeddings.llamacpp import LlamaCppEmbeddings
from langchaincoexpert.embeddings.localai import LocalAIEmbeddings
from langchaincoexpert.embeddings.minimax import MiniMaxEmbeddings
from langchaincoexpert.embeddings.mlflow_gateway import MlflowAIGatewayEmbeddings
from langchaincoexpert.embeddings.modelscope_hub import ModelScopeEmbeddings
from langchaincoexpert.embeddings.mosaicml import MosaicMLInstructorEmbeddings
from langchaincoexpert.embeddings.nlpcloud import NLPCloudEmbeddings
from langchaincoexpert.embeddings.octoai_embeddings import OctoAIEmbeddings
from langchaincoexpert.embeddings.ollama import OllamaEmbeddings
from langchaincoexpert.embeddings.openai import OpenAIEmbeddings
from langchaincoexpert.embeddings.sagemaker_endpoint import SagemakerEndpointEmbeddings
from langchaincoexpert.embeddings.self_hosted import SelfHostedEmbeddings
from langchaincoexpert.embeddings.self_hosted_hugging_face import (
    SelfHostedHuggingFaceEmbeddings,
    SelfHostedHuggingFaceInstructEmbeddings,
)
from langchaincoexpert.embeddings.sentence_transformer import SentenceTransformerEmbeddings
from langchaincoexpert.embeddings.spacy_embeddings import SpacyEmbeddings
from langchaincoexpert.embeddings.tensorflow_hub import TensorflowHubEmbeddings
from langchaincoexpert.embeddings.vertexai import VertexAIEmbeddings
from langchaincoexpert.embeddings.xinference import XinferenceEmbeddings

logger = logging.getLogger(__name__)

__all__ = [
    "OpenAIEmbeddings",
    "CacheBackedEmbeddings",
    "ClarifaiEmbeddings",
    "CohereEmbeddings",
    "ElasticsearchEmbeddings",
    "HuggingFaceEmbeddings",
    "HuggingFaceInferenceAPIEmbeddings",
    "JinaEmbeddings",
    "LlamaCppEmbeddings",
    "HuggingFaceHubEmbeddings",
    "MlflowAIGatewayEmbeddings",
    "ModelScopeEmbeddings",
    "TensorflowHubEmbeddings",
    "SagemakerEndpointEmbeddings",
    "HuggingFaceInstructEmbeddings",
    "MosaicMLInstructorEmbeddings",
    "SelfHostedEmbeddings",
    "SelfHostedHuggingFaceEmbeddings",
    "SelfHostedHuggingFaceInstructEmbeddings",
    "FakeEmbeddings",
    "DeterministicFakeEmbedding",
    "AlephAlphaAsymmetricSemanticEmbedding",
    "AlephAlphaSymmetricSemanticEmbedding",
    "SentenceTransformerEmbeddings",
    "GooglePalmEmbeddings",
    "MiniMaxEmbeddings",
    "VertexAIEmbeddings",
    "BedrockEmbeddings",
    "DeepInfraEmbeddings",
    "EdenAiEmbeddings",
    "DashScopeEmbeddings",
    "EmbaasEmbeddings",
    "OctoAIEmbeddings",
    "SpacyEmbeddings",
    "NLPCloudEmbeddings",
    "GPT4AllEmbeddings",
    "XinferenceEmbeddings",
    "LocalAIEmbeddings",
    "AwaEmbeddings",
    "HuggingFaceBgeEmbeddings",
    "ErnieEmbeddings",
    "OllamaEmbeddings",
    "QianfanEmbeddingsEndpoint",
]


# TODO: this is in here to maintain backwards compatibility
class HypotheticalDocumentEmbedder:
    def __init__(self, *args: Any, **kwargs: Any):
        logger.warning(
            "Using a deprecated class. Please use "
            "`from langchaincoexpert.chains import HypotheticalDocumentEmbedder` instead"
        )
        from langchaincoexpert.chains.hyde.base import HypotheticalDocumentEmbedder as H

        return H(*args, **kwargs)  # type: ignore

    @classmethod
    def from_llm(cls, *args: Any, **kwargs: Any) -> Any:
        logger.warning(
            "Using a deprecated class. Please use "
            "`from langchaincoexpert.chains import HypotheticalDocumentEmbedder` instead"
        )
        from langchaincoexpert.chains.hyde.base import HypotheticalDocumentEmbedder as H

        return H.from_llm(*args, **kwargs)



"""**Utilities** are the integrations with third-part systems and packages.

Other langchaincoexpert classes use **Utilities** to interact with third-part systems
and packages.
"""
from langchaincoexpert.utilities.alpha_vantage import AlphaVantageAPIWrapper
from langchaincoexpert.utilities.apify import ApifyWrapper
from langchaincoexpert.utilities.arxiv import ArxivAPIWrapper
from langchaincoexpert.utilities.awslambda import LambdaWrapper
from langchaincoexpert.utilities.bash import BashProcess
from langchaincoexpert.utilities.bibtex import BibtexparserWrapper
from langchaincoexpert.utilities.bing_search import BingSearchAPIWrapper
from langchaincoexpert.utilities.brave_search import BraveSearchWrapper
from langchaincoexpert.utilities.duckduckgo_search import DuckDuckGoSearchAPIWrapper
from langchaincoexpert.utilities.golden_query import GoldenQueryAPIWrapper
from langchaincoexpert.utilities.google_places_api import GooglePlacesAPIWrapper
from langchaincoexpert.utilities.google_search import GoogleSearchAPIWrapper
from langchaincoexpert.utilities.google_serper import GoogleSerperAPIWrapper
from langchaincoexpert.utilities.graphql import GraphQLAPIWrapper
from langchaincoexpert.utilities.jira import JiraAPIWrapper
from langchaincoexpert.utilities.max_compute import MaxComputeAPIWrapper
from langchaincoexpert.utilities.metaphor_search import MetaphorSearchAPIWrapper
from langchaincoexpert.utilities.openweathermap import OpenWeatherMapAPIWrapper
from langchaincoexpert.utilities.portkey import Portkey
from langchaincoexpert.utilities.powerbi import PowerBIDataset
from langchaincoexpert.utilities.pubmed import PubMedAPIWrapper
from langchaincoexpert.utilities.python import PythonREPL
from langchaincoexpert.utilities.requests import Requests, RequestsWrapper, TextRequestsWrapper
from langchaincoexpert.utilities.scenexplain import SceneXplainAPIWrapper
from langchaincoexpert.utilities.searx_search import SearxSearchWrapper
from langchaincoexpert.utilities.serpapi import SerpAPIWrapper
from langchaincoexpert.utilities.spark_sql import SparkSQL
from langchaincoexpert.utilities.sql_database import SQLDatabase
from langchaincoexpert.utilities.tensorflow_datasets import TensorflowDatasets
from langchaincoexpert.utilities.twilio import TwilioAPIWrapper
from langchaincoexpert.utilities.wikipedia import WikipediaAPIWrapper
from langchaincoexpert.utilities.wolfram_alpha import WolframAlphaAPIWrapper
from langchaincoexpert.utilities.zapier import ZapierNLAWrapper

__all__ = [
    "AlphaVantageAPIWrapper",
    "ApifyWrapper",
    "ArxivAPIWrapper",
    "BashProcess",
    "BibtexparserWrapper",
    "BingSearchAPIWrapper",
    "BraveSearchWrapper",
    "DuckDuckGoSearchAPIWrapper",
    "GoldenQueryAPIWrapper",
    "GooglePlacesAPIWrapper",
    "GoogleSearchAPIWrapper",
    "GoogleSerperAPIWrapper",
    "GraphQLAPIWrapper",
    "JiraAPIWrapper",
    "LambdaWrapper",
    "MaxComputeAPIWrapper",
    "MetaphorSearchAPIWrapper",
    "OpenWeatherMapAPIWrapper",
    "Portkey",
    "PowerBIDataset",
    "PubMedAPIWrapper",
    "PythonREPL",
    "Requests",
    "RequestsWrapper",
    "SQLDatabase",
    "SceneXplainAPIWrapper",
    "SearxSearchWrapper",
    "SerpAPIWrapper",
    "SparkSQL",
    "TensorflowDatasets",
    "TextRequestsWrapper",
    "TextRequestsWrapper",
    "TwilioAPIWrapper",
    "WikipediaAPIWrapper",
    "WolframAlphaAPIWrapper",
    "ZapierNLAWrapper",
]



from __future__ import annotations

from typing import TYPE_CHECKING, List, Literal, Optional

from langchaincoexpert.agents.agent_toolkits.base import BaseToolkit
from langchaincoexpert.pydantic_v1 import root_validator
from langchaincoexpert.tools import BaseTool
from langchaincoexpert.tools.ainetwork.app import AINAppOps
from langchaincoexpert.tools.ainetwork.owner import AINOwnerOps
from langchaincoexpert.tools.ainetwork.rule import AINRuleOps
from langchaincoexpert.tools.ainetwork.transfer import AINTransfer
from langchaincoexpert.tools.ainetwork.utils import authenticate
from langchaincoexpert.tools.ainetwork.value import AINValueOps

if TYPE_CHECKING:
    from ain.ain import Ain


class AINetworkToolkit(BaseToolkit):
    """Toolkit for interacting with AINetwork Blockchain."""

    network: Optional[Literal["mainnet", "testnet"]] = "testnet"
    interface: Optional[Ain] = None

    @root_validator(pre=True)
    def set_interface(cls, values: dict) -> dict:
        if not values.get("interface"):
            values["interface"] = authenticate(network=values.get("network", "testnet"))
        return values

    class Config:
        """Pydantic config."""

        validate_all = True
        arbitrary_types_allowed = True

    def get_tools(self) -> List[BaseTool]:
        """Get the tools in the toolkit."""
        return [
            AINAppOps(),
            AINOwnerOps(),
            AINRuleOps(),
            AINTransfer(),
            AINValueOps(),
        ]



# flake8: noqa
from langchaincoexpert.chains.prompt_selector import ConditionalPromptSelector, is_chat_model
from langchaincoexpert.prompts.chat import (
    AIMessagePromptTemplate,
    ChatPromptTemplate,
    HumanMessagePromptTemplate,
    SystemMessagePromptTemplate,
)
from langchaincoexpert.prompts.prompt import PromptTemplate

DEFAULT_REFINE_PROMPT_TMPL = (
    "The original question is as follows: {question}\n"
    "We have provided an existing answer: {existing_answer}\n"
    "We have the opportunity to refine the existing answer "
    "(only if needed) with some more context below.\n"
    "------------\n"
    "{context_str}\n"
    "------------\n"
    "Given the new context, refine the original answer to better "
    "answer the question. "
    "If the context isn't useful, return the original answer."
)
DEFAULT_REFINE_PROMPT = PromptTemplate.from_template(DEFAULT_REFINE_PROMPT_TMPL)

refine_template = (
    "We have the opportunity to refine the existing answer "
    "(only if needed) with some more context below.\n"
    "------------\n"
    "{context_str}\n"
    "------------\n"
    "Given the new context, refine the original answer to better "
    "answer the question. "
    "If the context isn't useful, return the original answer."
)
CHAT_REFINE_PROMPT = ChatPromptTemplate.from_messages(
    [("human", "{question}"), ("ai", "{existing_answer}"), ("human", "refine_template")]
)
REFINE_PROMPT_SELECTOR = ConditionalPromptSelector(
    default_prompt=DEFAULT_REFINE_PROMPT,
    conditionals=[(is_chat_model, CHAT_REFINE_PROMPT)],
)


DEFAULT_TEXT_QA_PROMPT_TMPL = (
    "Context information is below. \n"
    "------------\n"
    "{context_str}\n"
    "------------\n"
    "Given the context information and not prior knowledge, "
    "answer the question: {question}\n"
)
DEFAULT_TEXT_QA_PROMPT = PromptTemplate.from_template(DEFAULT_TEXT_QA_PROMPT_TMPL)

chat_qa_prompt_template = (
    "Context information is below.\n"
    "------------\n"
    "{context_str}\n"
    "------------\n"
    "Given the context information and not prior knowledge, "
    "answer any questions"
)
CHAT_QUESTION_PROMPT = ChatPromptTemplate.from_messages(
    [("system", chat_qa_prompt_template), ("human", "{question}")]
)
QUESTION_PROMPT_SELECTOR = ConditionalPromptSelector(
    default_prompt=DEFAULT_TEXT_QA_PROMPT,
    conditionals=[(is_chat_model, CHAT_QUESTION_PROMPT)],
)



"""**Tools** are classes that an Agent uses to interact with the world.

Each tool has a **description**. Agent uses the description to choose the right
tool for the job.

**Class hierarchy:**

.. code-block::

    ToolMetaclass --> BaseTool --> <name>Tool  # Examples: AIPluginTool, BaseGraphQLTool
                                   <name>      # Examples: BraveSearch, HumanInputRun

**Main helpers:**

.. code-block::

    CallbackManagerForToolRun, AsyncCallbackManagerForToolRun
"""

from langchaincoexpert.tools.ainetwork.app import AINAppOps
from langchaincoexpert.tools.ainetwork.owner import AINOwnerOps
from langchaincoexpert.tools.ainetwork.rule import AINRuleOps
from langchaincoexpert.tools.ainetwork.transfer import AINTransfer
from langchaincoexpert.tools.ainetwork.value import AINValueOps
from langchaincoexpert.tools.arxiv.tool import ArxivQueryRun
from langchaincoexpert.tools.azure_cognitive_services import (
    AzureCogsFormRecognizerTool,
    AzureCogsImageAnalysisTool,
    AzureCogsSpeech2TextTool,
    AzureCogsText2SpeechTool,
)
from langchaincoexpert.tools.base import BaseTool, StructuredTool, Tool, tool
from langchaincoexpert.tools.bing_search.tool import BingSearchResults, BingSearchRun
from langchaincoexpert.tools.brave_search.tool import BraveSearch
from langchaincoexpert.tools.convert_to_openai import format_tool_to_openai_function
from langchaincoexpert.tools.ddg_search.tool import DuckDuckGoSearchResults, DuckDuckGoSearchRun
from langchaincoexpert.tools.edenai import (
    EdenAiExplicitImageTool,
    EdenAiObjectDetectionTool,
    EdenAiParsingIDTool,
    EdenAiParsingInvoiceTool,
    EdenAiSpeechToTextTool,
    EdenAiTextModerationTool,
    EdenAiTextToSpeechTool,
    EdenaiTool,
)
from langchaincoexpert.tools.eleven_labs.text2speech import ElevenLabsText2SpeechTool
from langchaincoexpert.tools.file_management import (
    CopyFileTool,
    DeleteFileTool,
    FileSearchTool,
    ListDirectoryTool,
    MoveFileTool,
    ReadFileTool,
    WriteFileTool,
)
from langchaincoexpert.tools.gmail import (
    GmailCreateDraft,
    GmailGetMessage,
    GmailGetThread,
    GmailSearch,
    GmailSendMessage,
)
from langchaincoexpert.tools.google_places.tool import GooglePlacesTool
from langchaincoexpert.tools.google_search.tool import GoogleSearchResults, GoogleSearchRun
from langchaincoexpert.tools.google_serper.tool import GoogleSerperResults, GoogleSerperRun
from langchaincoexpert.tools.graphql.tool import BaseGraphQLTool
from langchaincoexpert.tools.human.tool import HumanInputRun
from langchaincoexpert.tools.ifttt import IFTTTWebhook
from langchaincoexpert.tools.interaction.tool import StdInInquireTool
from langchaincoexpert.tools.jira.tool import JiraAction
from langchaincoexpert.tools.json.tool import JsonGetValueTool, JsonListKeysTool
from langchaincoexpert.tools.metaphor_search import MetaphorSearchResults
from langchaincoexpert.tools.office365.create_draft_message import O365CreateDraftMessage
from langchaincoexpert.tools.office365.events_search import O365SearchEvents
from langchaincoexpert.tools.office365.messages_search import O365SearchEmails
from langchaincoexpert.tools.office365.send_event import O365SendEvent
from langchaincoexpert.tools.office365.send_message import O365SendMessage
from langchaincoexpert.tools.office365.utils import authenticate
from langchaincoexpert.tools.openapi.utils.api_models import APIOperation
from langchaincoexpert.tools.openapi.utils.openapi_utils import OpenAPISpec
from langchaincoexpert.tools.openweathermap.tool import OpenWeatherMapQueryRun
from langchaincoexpert.tools.playwright import (
    ClickTool,
    CurrentWebPageTool,
    ExtractHyperlinksTool,
    ExtractTextTool,
    GetElementsTool,
    NavigateBackTool,
    NavigateTool,
)
from langchaincoexpert.tools.plugin import AIPluginTool
from langchaincoexpert.tools.powerbi.tool import (
    InfoPowerBITool,
    ListPowerBITool,
    QueryPowerBITool,
)
from langchaincoexpert.tools.pubmed.tool import PubmedQueryRun
from langchaincoexpert.tools.python.tool import PythonAstREPLTool, PythonREPLTool
from langchaincoexpert.tools.requests.tool import (
    BaseRequestsTool,
    RequestsDeleteTool,
    RequestsGetTool,
    RequestsPatchTool,
    RequestsPostTool,
    RequestsPutTool,
)
from langchaincoexpert.tools.scenexplain.tool import SceneXplainTool
from langchaincoexpert.tools.searx_search.tool import SearxSearchResults, SearxSearchRun
from langchaincoexpert.tools.shell.tool import ShellTool
from langchaincoexpert.tools.sleep.tool import SleepTool
from langchaincoexpert.tools.spark_sql.tool import (
    BaseSparkSQLTool,
    InfoSparkSQLTool,
    ListSparkSQLTool,
    QueryCheckerTool,
    QuerySparkSQLTool,
)
from langchaincoexpert.tools.sql_database.tool import (
    BaseSQLDatabaseTool,
    InfoSQLDatabaseTool,
    ListSQLDatabaseTool,
    QuerySQLCheckerTool,
    QuerySQLDataBaseTool,
)
from langchaincoexpert.tools.steamship_image_generation import SteamshipImageGenerationTool
from langchaincoexpert.tools.vectorstore.tool import (
    VectorStoreQATool,
    VectorStoreQAWithSourcesTool,
)
from langchaincoexpert.tools.wikipedia.tool import WikipediaQueryRun
from langchaincoexpert.tools.wolfram_alpha.tool import WolframAlphaQueryRun
from langchaincoexpert.tools.youtube.search import YouTubeSearchTool
from langchaincoexpert.tools.zapier.tool import ZapierNLAListActions, ZapierNLARunAction

__all__ = [
    "AINAppOps",
    "AINOwnerOps",
    "AINRuleOps",
    "AINTransfer",
    "AINValueOps",
    "AIPluginTool",
    "APIOperation",
    "ArxivQueryRun",
    "AzureCogsFormRecognizerTool",
    "AzureCogsImageAnalysisTool",
    "AzureCogsSpeech2TextTool",
    "AzureCogsText2SpeechTool",
    "BaseGraphQLTool",
    "BaseRequestsTool",
    "BaseSQLDatabaseTool",
    "BaseSparkSQLTool",
    "BaseTool",
    "BingSearchResults",
    "BingSearchRun",
    "BraveSearch",
    "ClickTool",
    "CopyFileTool",
    "CurrentWebPageTool",
    "DeleteFileTool",
    "DuckDuckGoSearchResults",
    "DuckDuckGoSearchRun",
    "EdenAiExplicitImageTool",
    "EdenAiObjectDetectionTool",
    "EdenAiParsingIDTool",
    "EdenAiParsingInvoiceTool",
    "EdenAiTextToSpeechTool",
    "EdenAiSpeechToTextTool",
    "EdenAiTextModerationTool",
    "EdenaiTool",
    "ElevenLabsText2SpeechTool",
    "ExtractHyperlinksTool",
    "ExtractTextTool",
    "FileSearchTool",
    "GetElementsTool",
    "GmailCreateDraft",
    "GmailGetMessage",
    "GmailGetThread",
    "GmailSearch",
    "GmailSendMessage",
    "GooglePlacesTool",
    "GoogleSearchResults",
    "GoogleSearchRun",
    "GoogleSerperResults",
    "GoogleSerperRun",
    "HumanInputRun",
    "IFTTTWebhook",
    "InfoPowerBITool",
    "InfoSQLDatabaseTool",
    "InfoSparkSQLTool",
    "JiraAction",
    "JsonGetValueTool",
    "JsonListKeysTool",
    "ListDirectoryTool",
    "ListPowerBITool",
    "ListSQLDatabaseTool",
    "ListSparkSQLTool",
    "MetaphorSearchResults",
    "MoveFileTool",
    "NavigateBackTool",
    "NavigateTool",
    "O365SearchEmails",
    "O365SearchEvents",
    "O365CreateDraftMessage",
    "O365SendMessage",
    "O365SendEvent",
    "authenticate",
    "OpenAPISpec",
    "OpenWeatherMapQueryRun",
    "PubmedQueryRun",
    "PythonAstREPLTool",
    "PythonREPLTool",
    "QueryCheckerTool",
    "QueryPowerBITool",
    "QuerySQLCheckerTool",
    "QuerySQLDataBaseTool",
    "QuerySparkSQLTool",
    "ReadFileTool",
    "RequestsDeleteTool",
    "RequestsGetTool",
    "RequestsPatchTool",
    "RequestsPostTool",
    "RequestsPutTool",
    "SceneXplainTool",
    "SearxSearchResults",
    "SearxSearchRun",
    "ShellTool",
    "SleepTool",
    "StdInInquireTool",
    "SteamshipImageGenerationTool",
    "StructuredTool",
    "Tool",
    "VectorStoreQATool",
    "VectorStoreQAWithSourcesTool",
    "WikipediaQueryRun",
    "WolframAlphaQueryRun",
    "WriteFileTool",
    "YouTubeSearchTool",
    "ZapierNLAListActions",
    "ZapierNLARunAction",
    "format_tool_to_openai_function",
    "tool",
]



from typing import List

from langchaincoexpert.callbacks.manager import CallbackManagerForRetrieverRun
from langchaincoexpert.schema import BaseRetriever, Document
from langchaincoexpert.utilities.arxiv import ArxivAPIWrapper


class ArxivRetriever(BaseRetriever, ArxivAPIWrapper):
    """`Arxiv` retriever.

    It wraps load() to get_relevant_documents().
    It uses all ArxivAPIWrapper arguments without any change.
    """

    def _get_relevant_documents(
        self, query: str, *, run_manager: CallbackManagerForRetrieverRun
    ) -> List[Document]:
        return self.load(query=query)
